==> xmlroff-0.3.3.tar.gz: xmlroff-0.3.3/xmlroff.c <==
/* Fo
 * xmlroff.c: Demonstration command line XSL formatter program
 *
 * Copyright (C) 2001-2004 Sun Microsystems
 *
 * $Id: xmlroff.c,v 1.16 2004/10/21 00:35:26 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include "config.h"
#include <stdlib.h>
#include <string.h>
#include <popt.h>
#include <libfo/fo-libfo.h>
#if ENABLE_GP
#include <libfo/fo-doc-gp.h>
#endif

static FoDoc *
init_fo_doc_gp (const gchar *out_file)
{
  FoDoc *fo_doc = NULL;
  GError *error = NULL;

#if ENABLE_GP
  fo_doc = fo_doc_gp_new ();

  fo_doc_open_file (fo_doc,
		    out_file,
		    &error);

  if (error != NULL)
    {
      g_critical ("%s:: %s",
		  g_quark_to_string (error->domain),
		  error->message);
      g_error_free (error);
      exit (1);
    }
#else
  g_critical ("Output using GNOME Print is not supported by this build of libfo.");
  exit (1);
#endif /* ENABLE_GP */

  return fo_doc;
}

int
main (int          argc,
      const char **argv)
{
  poptContext optCon;   /* context for parsing command-line options */
  FoLibfoContext *libfo_context;
  FoXmlDoc *xml_doc = NULL;
  FoXmlDoc *result_tree = NULL;
  FoXsltTransformer *fo_xslt_transformer = NULL;
  FoXslFormatter *fo_xsl_formatter;
  FoDoc *fo_doc = NULL;
  GError *error = NULL;
  gchar *out_file = "layout.pdf";
  const gchar *xml_file = NULL;
  const gchar *xslt_file = NULL;
  const gchar *backend_string = NULL;
  const gchar *warning_string = NULL;
  const gchar *embed_string = NULL;
  FoEnumFontEmbed embed_mode = FO_ENUM_FONT_EMBED_INVALID;
  FoDebugFlag debug_mode = FO_DEBUG_NONE;
  gint catalog = 0;
  gint novalid = 0;
  gint version = 0;
  gboolean popt_error = FALSE;

  const struct poptOption optionsTable[] = {
    { "out-file",
      'o',
      POPT_ARG_STRING,
      &out_file,
      0,
      _("Output file"),
      _("filename")
    },
    { "debug",
      'd',
      POPT_ARG_INT,
      &debug_mode,
      0,
      _("Debug mode"),
      _("integer")
    },
    /*
    { "embed",
      '\0',
      POPT_ARG_STRING,
      &embed_string,
      0,
      _("Font embedding mode"),
      _("{none|nonbase|all}")
    },
    */
    { "catalogs",
      '\0',
      POPT_ARG_NONE,
      &catalog,
      0,
      _("Use SGML catalogs from $SGML_CATALOG_FILES instead of XML catalogs from file:///etc/xml/catalog"),
      NULL
    },
    { "novalid",
      '\0',
      POPT_ARG_NONE,
      &novalid,
      0,
      _("Skip the DTD loading phase"),
      NULL
    },
    { "version",
      'v',
      POPT_ARG_NONE,
      &version,
      0,
      _("Print version number"),
      NULL
    },
    { "backend",
      '\0',
      POPT_ARG_STRING,
      &backend_string,
      0,
      _("Backend to use"),
      _("{gp}")
    },
    { "warn",
      'w',
      POPT_ARG_STRING,
      &warning_string,
      0,
      _("Warning"),
      _("{property|fo|both}")
    },
    POPT_AUTOHELP
    { NULL, '\0', 0, NULL, 0, NULL, NULL}
  };

  optCon = poptGetContext (NULL, argc, argv, optionsTable, 0);
  /* FIXME: Support xml-stylesheet PI in xml-file */
  poptSetOtherOptionHelp (optCon, _("xml-file [stylesheet]"));

  poptGetNextOpt (optCon);

  fo_libfo_init ();
  libfo_context = fo_libfo_context_new ();

  if (version != 0)
    {
      g_print (_("%s\nSubmit bug reports to %s\n"),
	       PACKAGE_STRING,
	       PACKAGE_BUGREPORT);
    }

  if (embed_string != NULL)
    {
      if (strcmp (embed_string, "none") == 0)
	{
	  embed_mode = FO_ENUM_FONT_EMBED_NONE;
	}
      else if (strcmp (embed_string, "nonbase") == 0)
	{
	  embed_mode = FO_ENUM_FONT_EMBED_NONBASE;
	}
      else if (strcmp (embed_string, "all") == 0)
	{
	  embed_mode = FO_ENUM_FONT_EMBED_ALL;
	}
      else
	{
	  popt_error = TRUE;
	}

      if (popt_error != TRUE)
	{
	  fo_libfo_context_set_font_embed (libfo_context,
					   embed_mode);
	}
    }


  if (novalid == 0)
    {
      fo_libfo_context_set_validation (libfo_context,
				       FALSE);
    }


  if (catalog != 0)
    {
      fo_libfo_context_set_sgml_catalogs (libfo_context,
					  TRUE);
    }

  if (backend_string == NULL)
    {
#if ENABLE_GP
      fo_doc = init_fo_doc_gp (out_file);
#else
      g_critical ("No output type is supported by this build of libfo.");
      popt_error = TRUE;
#endif /* ENABLE_GP */
    }
  else if (strcmp (backend_string, "gp") == 0)
    {
      fo_doc = init_fo_doc_gp (out_file);
    }
  else
    {
      g_critical ("Unrecognised output type: '%s'", backend_string);
      popt_error = TRUE;
    }

  if (debug_mode != FO_DEBUG_NONE)
    {
      fo_libfo_context_set_debug_mode (libfo_context,
				       debug_mode);
    }

  if (warning_string != NULL)
    {
      FoWarningFlag warning_mode = FO_WARNING_NONE;

      if (strcmp(warning_string, "property") == 0)
	{
	  warning_mode = FO_WARNING_PROPERTY;
	}
      else if (strcmp(warning_string, "fo") == 0)
	{
	  warning_mode = FO_WARNING_FO;
	}
      else if (strcmp(warning_string, "both") == 0)
	{
	  warning_mode = FO_WARNING_FO | FO_WARNING_PROPERTY;
	}
      else
	{
	  g_warning ("Unknown 'warn' option: %s", warning_string);
	}

      fo_libfo_context_set_warning_mode (libfo_context,
					 warning_mode);
    }

  if (poptPeekArg (optCon))
    {
      xml_file = poptGetArg (optCon);
    }
  else
    {
      if (version != 0)
	{
	  exit (0);
	}
      else
	{
	  popt_error = TRUE;
	}
    }

  if (poptPeekArg (optCon))
    {
      xslt_file = poptGetArg (optCon);
    }

  if (poptPeekArg (optCon))
    {
      popt_error = TRUE;
    }

  if (popt_error == TRUE)
    {
      poptPrintUsage(optCon, stderr, 0);
      exit (1);
    }

  if (error != NULL)
    {
      g_critical ("%s:: %s",
		  g_quark_to_string (error->domain),
		  error->message);
      g_error_free (error);
      exit (1);
    }

  if (xslt_file != NULL)
    {
      xml_doc = fo_xml_doc_new ();
      fo_xml_doc_set_filename (xml_doc,
			       xml_file);

      fo_xml_doc_parse (xml_doc,
			libfo_context,
			&error);

      if (error != NULL)
	{
	  g_critical ("%s:: %s",
		      g_quark_to_string (error->domain),
		      error->message);
	  g_error_free (error);
	  exit (1);
	}

      fo_xslt_transformer = fo_xslt_transformer_new ();
      fo_xslt_transformer_set_filename (fo_xslt_transformer,
					xslt_file);
      fo_xslt_transformer_set_xml_doc (fo_xslt_transformer,
				       xml_doc);
      fo_xslt_transformer_transform (fo_xslt_transformer,
				    libfo_context,
				    &error);
      if (error != NULL)
	{
	  g_critical ("%s:: %s",
		      g_quark_to_string (error->domain),
		      error->message);
	  g_error_free (error);
	  exit (1);
	}

      fo_xml_doc_unref (xml_doc);
      result_tree =
	fo_xml_doc_ref (fo_xslt_transformer_get_result (fo_xslt_transformer));
      fo_xslt_transformer_unref (fo_xslt_transformer);
    }
  else
    {
      result_tree = fo_xml_doc_new ();
      fo_xml_doc_set_filename (result_tree,
			       xml_file);
      fo_xml_doc_parse (result_tree,
			libfo_context,
			&error);

      if (error != NULL)
	{
	  g_critical ("%s:: %s",
		      g_quark_to_string (error->domain),
		      error->message);
	  g_error_free (error);
	  exit (1);
	}
    }

  fo_xsl_formatter = fo_xsl_formatter_new ();

  fo_xsl_formatter_set_result_tree (fo_xsl_formatter,
				    result_tree);

  fo_xsl_formatter_set_fo_doc (fo_xsl_formatter,
			       fo_doc);

  fo_xsl_formatter_format (fo_xsl_formatter,
			   libfo_context,
			   &error);
  if (error != NULL)
    {
      g_message ("Error: %s", error->message);
      exit (1);
    }

  fo_xsl_formatter_draw (fo_xsl_formatter,
			 libfo_context,
			 &error);

  if (error != NULL)
    {
      g_message ("Error: %s", error->message);
      exit (1);
    }

  g_object_unref (fo_xsl_formatter);
  g_object_unref (fo_doc);
  fo_libfo_shutdown ();

  poptFreeContext (optCon);

  return(0);
}

==> xmlroff-0.3.7.tar.gz: xmlroff-0.3.7/xmlroff.c <==
/* Fo
 * xmlroff.c: Demonstration command line XSL formatter program
 *
 * Copyright (C) 2001-2004 Sun Microsystems
 *
 * $Id: xmlroff.c,v 1.17 2005/06/27 00:14:21 tonygraham Exp $
 *
 * See Copying for the status of this software.
 */

#include "config.h"
#include <stdlib.h>
#include <string.h>
#include <popt.h>
#include <libfo/fo-libfo.h>
#if ENABLE_GP
#include <libfo/fo-doc-gp.h>
#endif

static FoDoc *
init_fo_doc_gp (const gchar    *out_file,
		FoLibfoContext *libfo_context)
{
  FoDoc *fo_doc = NULL;
  GError *error = NULL;

#if ENABLE_GP
  fo_doc = fo_doc_gp_new ();

  fo_doc_open_file (fo_doc,
		    out_file,
		    libfo_context,
		    &error);

  if (error != NULL)
    {
      g_critical ("%s:: %s",
		  g_quark_to_string (error->domain),
		  error->message);
      g_error_free (error);
      exit (1);
    }
#else
  g_critical ("Output using GNOME Print is not supported by this build of libfo.");
  exit (1);
#endif /* ENABLE_GP */

  return fo_doc;
}

int
main (int          argc,
      const char **argv)
{
  poptContext optCon;   /* context for parsing command-line options */
  FoLibfoContext *libfo_context;
  FoXmlDoc *xml_doc = NULL;
  FoXmlDoc *result_tree = NULL;
  FoXsltTransformer *fo_xslt_transformer = NULL;
  FoXslFormatter *fo_xsl_formatter;
  FoDoc *fo_doc = NULL;
  GError *error = NULL;
  gchar *out_file = "layout.pdf";
  const gchar *xml_file = NULL;
  const gchar *xslt_file = NULL;
  const gchar *backend_string = NULL;
  const gchar *format_string = NULL;
  const gchar *warning_string = NULL;
  const gchar *embed_string = NULL;
  FoEnumFontEmbed embed_mode = FO_ENUM_FONT_EMBED_INVALID;
  FoEnumFormat format_mode = FO_ENUM_FORMAT_UNKNOWN;
  FoDebugFlag debug_mode = FO_DEBUG_NONE;
  gint catalog = 0;
  gint novalid = 0;
  gint version = 0;
  gboolean popt_error = FALSE;

  const struct poptOption optionsTable[] = {
    { "out-file",
      'o',
      POPT_ARG_STRING,
      &out_file,
      0,
      _("Output file"),
      _("filename")
    },
    { "debug",
      'd',
      POPT_ARG_INT,
      &debug_mode,
      0,
      _("Debug mode"),
      _("integer")
    },
    /*
    { "embed",
      '\0',
      POPT_ARG_STRING,
      &embed_string,
      0,
      _("Font embedding mode"),
      _("{none|nonbase|all}")
    },
    */
    { "catalogs",
      '\0',
      POPT_ARG_NONE,
      &catalog,
      0,
      _("Use SGML catalogs from $SGML_CATALOG_FILES instead of XML catalogs from file:///etc/xml/catalog"),
      NULL
    },
    { "novalid",
      '\0',
      POPT_ARG_NONE,
      &novalid,
      0,
      _("Skip the DTD loading phase"),
      NULL
    },
    { "version",
      'v',
      POPT_ARG_NONE,
      &version,
      0,
      _("Print version number"),
      NULL
    },
    { "backend",
      '\0',
      POPT_ARG_STRING,
      &backend_string,
      0,
      _("Backend to use"),
      _("{gp}")
    },
    { "format",
      '\0',
      POPT_ARG_STRING,
      &format_string,
      0,
      _("Format of output file"),
      _("{auto|pdf|postscript|svg}")
    },
    { "warn",
      'w',
      POPT_ARG_STRING,
      &warning_string,
      0,
      _("Warning"),
      _("{property|fo|both}")
    },
    POPT_AUTOHELP
    { NULL, '\0', 0, NULL, 0, NULL, NULL}
  };

  optCon = poptGetContext (NULL, argc, argv, optionsTable, 0);
  /* FIXME: Support xml-stylesheet PI in xml-file */
  poptSetOtherOptionHelp (optCon, _("xml-file [stylesheet]"));

  poptGetNextOpt (optCon);

  fo_libfo_init ();
  libfo_context = fo_libfo_context_new ();

  if (version != 0)
    {
      g_print (_("%s\nSubmit bug reports to %s\n"),
	       PACKAGE_STRING,
	       PACKAGE_BUGREPORT);
    }

  if (embed_string != NULL)
    {
      if (strcmp (embed_string, "none") == 0)
	{
	  embed_mode = FO_ENUM_FONT_EMBED_NONE;
	}
      else if (strcmp (embed_string, "nonbase") == 0)
	{
	  embed_mode = FO_ENUM_FONT_EMBED_NONBASE;
	}
      else if (strcmp (embed_string, "all") == 0)
	{
	  embed_mode = FO_ENUM_FONT_EMBED_ALL;
	}
      else
	{
	  popt_error = TRUE;
	}

      if (popt_error != TRUE)
	{
	  fo_libfo_context_set_font_embed (libfo_context,
					   embed_mode);
	}
    }


  if (novalid == 0)
    {
      fo_libfo_context_set_validation (libfo_context,
				       FALSE);
    }


  if (catalog != 0)
    {
      fo_libfo_context_set_sgml_catalogs (libfo_context,
					  TRUE);
    }

  /* Need to do 'format' before 'backend'. */
  if ((format_string == NULL) ||
      (strcmp (format_string, "auto") == 0))
    {
      format_mode = FO_ENUM_FORMAT_AUTO;
    }
  else if (strcmp (format_string, "pdf") == 0)
    {
      format_mode = FO_ENUM_FORMAT_PDF;
    }
  else if (strcmp (format_string, "postscript") == 0)
    {
      format_mode = FO_ENUM_FORMAT_POSTSCRIPT;
    }
  else if (strcmp (format_string, "svg") == 0)
    {
      format_mode = FO_ENUM_FORMAT_SVG;
    }
  else
    {
      popt_error = TRUE;
    }

  if (popt_error != TRUE)
    {
      fo_libfo_context_set_format (libfo_context,
				   format_mode);
    }

  if (backend_string == NULL)
    {
#if ENABLE_GP
      fo_doc = init_fo_doc_gp (out_file, libfo_context);
#else
      g_critical ("No output type is supported by this build of libfo.");
      popt_error = TRUE;
#endif /* ENABLE_GP */
    }
  else if (strcmp (backend_string, "gp") == 0)
    {
      fo_doc = init_fo_doc_gp (out_file, libfo_context);
    }
  else
    {
      g_critical ("Unrecognised output type: '%s'", backend_string);
      popt_error = TRUE;
    }

  if (debug_mode != FO_DEBUG_NONE)
    {
      fo_libfo_context_set_debug_mode (libfo_context,
				       debug_mode);
    }

  if (warning_string != NULL)
    {
      FoWarningFlag warning_mode = FO_WARNING_NONE;

      if (strcmp(warning_string, "property") == 0)
	{
	  warning_mode = FO_WARNING_PROPERTY;
	}
      else if (strcmp(warning_string, "fo") == 0)
	{
	  warning_mode = FO_WARNING_FO;
	}
      else if (strcmp(warning_string, "both") == 0)
	{
	  warning_mode = FO_WARNING_FO | FO_WARNING_PROPERTY;
	}
      else
	{
	  g_warning ("Unknown 'warn' option: %s", warning_string);
	}

      fo_libfo_context_set_warning_mode (libfo_context,
					 warning_mode);
    }

  if (poptPeekArg (optCon))
    {
      xml_file = poptGetArg (optCon);
    }
  else
    {
      if (version != 0)
	{
	  exit (0);
	}
      else
	{
	  popt_error = TRUE;
	}
    }

  if (poptPeekArg (optCon))
    {
      xslt_file = poptGetArg (optCon);
    }

  if (poptPeekArg (optCon))
    {
      popt_error = TRUE;
    }

  if (popt_error == TRUE)
    {
      poptPrintUsage(optCon, stderr, 0);
      exit (1);
    }

  if (error != NULL)
    {
      g_critical ("%s:: %s",
		  g_quark_to_string (error->domain),
		  error->message);
      g_error_free (error);
      exit (1);
    }

  if (xslt_file != NULL)
    {
      xml_doc = fo_xml_doc_new ();
      fo_xml_doc_set_filename (xml_doc,
			       xml_file);

      fo_xml_doc_parse (xml_doc,
			libfo_context,
			&error);

      if (error != NULL)
	{
	  g_critical ("%s:: %s",
		      g_quark_to_string (error->domain),
		      error->message);
	  g_error_free (error);
	  exit (1);
	}

      fo_xslt_transformer = fo_xslt_transformer_new ();
      fo_xslt_transformer_set_filename (fo_xslt_transformer,
					xslt_file);
      fo_xslt_transformer_set_xml_doc (fo_xslt_transformer,
				       xml_doc);
      fo_xslt_transformer_transform (fo_xslt_transformer,
				    libfo_context,
				    &error);
      if (error != NULL)
	{
	  g_critical ("%s:: %s",
		      g_quark_to_string (error->domain),
		      error->message);
	  g_error_free (error);
	  exit (1);
	}

      fo_xml_doc_unref (xml_doc);
      result_tree =
	fo_xml_doc_ref (fo_xslt_transformer_get_result (fo_xslt_transformer));
      fo_xslt_transformer_unref (fo_xslt_transformer);
    }
  else
    {
      result_tree = fo_xml_doc_new ();
      fo_xml_doc_set_filename (result_tree,
			       xml_file);
      fo_xml_doc_parse (result_tree,
			libfo_context,
			&error);

      if (error != NULL)
	{
	  g_critical ("%s:: %s",
		      g_quark_to_string (error->domain),
		      error->message);
	  g_error_free (error);
	  exit (1);
	}
    }

  fo_xsl_formatter = fo_xsl_formatter_new ();

  fo_xsl_formatter_set_result_tree (fo_xsl_formatter,
				    result_tree);

  fo_xsl_formatter_set_fo_doc (fo_xsl_formatter,
			       fo_doc);

  fo_xsl_formatter_format (fo_xsl_formatter,
			   libfo_context,
			   &error);
  if (error != NULL)
    {
      g_message ("Error: %s", error->message);
      exit (1);
    }

  fo_xsl_formatter_draw (fo_xsl_formatter,
			 libfo_context,
			 &error);

  if (error != NULL)
    {
      g_message ("Error: %s", error->message);
      exit (1);
    }

  g_object_unref (fo_xsl_formatter);
  g_object_unref (fo_doc);
  fo_libfo_shutdown ();

  poptFreeContext (optCon);

  return(0);
}
